---
title: 'Tutorial Part 2: Modifying and Adding Detectors '
---

  * [Introduction](#introduction)
  * [How to build a detector from scratch](#how-to-build-a-detector-from-scratch)
    + [Compiling a new detector](#compiling-a-new-detector)
    + [Building the geometry](#building-the-geometry)
      - [Compact detector description entry element](#compact-detector-description-entry-element)
      - [Geometry Construction](#geometry-construction)
        * [XML Parsing Tip : Provide good default values](#xml-parsing-tip--provide-good-default-values)
      - [Critical parts of build_detector](#critical-parts-of-build_detector)
  * [The Readout and Bit Fields](#the-readout-and-bit-fields)
  * [Simple Reconstruction Overview of scripts](#simple-reconstruction-overview-of-scripts)
    + [Dependencies](#dependencies)
    + [Running the scripts](#running-the-scripts)
    + [To Do](#to-do)

## Setup 

Note all these commands assume you are in an `eic-shell` singularity session.

### Clone part2 repo

```bash
git clone https://eicweb.phy.anl.gov/EIC/tutorials/ip6_tutorial_2.git part2
cd part2
```

### Compile the detector library

```bash
mkdir build
cmake ../. -DCMAKE_INSTALL_PREFIX=../../local 
make -j4 install
```


How to build a detector from scratch
------------------------------------

Building a new (generic) detector using dd4hep is rather straight forward if we 
make the following assumptions.

1. We will use the built-in sensitive detectors
2. We will use the dd4pod data model (i.e. use `npsim`)


### Compiling a new detector

For this tutorial we will build a more detailed GEM detector.
We will discuss the detector built in the source file
`src/TrapEndcapTracker_geo.cpp`.

In the `CMakeLists.txt` it adds all cpp files, otherwise we would do
```bash
dd4hep_add_plugin(${a_lib_name} SOURCES
  src/TrapEndcapTracker_geo.cpp
  ...
  )
```

###  Building the geometry (cpp + xml)

The work of constructing the detector is done in a static function (here called 
`create_detector`) that is registered using the DD4hep plugin macro 
`DECLARE_DETELEMENT`.

```cpp
static Ref_t create_detector(Detector& dtor, xml_h e, SensitiveDetector sens)
{
  xml_det_t   x_det     = e;
  Material    air       = dtor.air();
...
}
DECLARE_DETELEMENT(MyGEMTrackerEndcap, build_detector)
```

The argument signature of the `create_detector` is:
- `Detector& dtor`: This handle provides the main hook to the detector tree (`dd4hep::Detector`).
- `xml_h e`: Handle to the XML `<detector>` tag associated with the detector in 
  the "compact" detector description file (more on this later). This provides 
  the mechanism for feeding in the run-time construction parameters. 
- `SensitiveDetector sens`: The sensitive detector to be assigned to the 
  sensitive volumes/elements of the detector. 

The DD4hep plugin macro `DECLARE_DETELEMENT(MyGEMTrackerEndcap, build_detector)` 
stamps out the necessary boiler plate code to register a new detector called 
`MyGEMTrackerEndcap` which is build by calling `create_detector`.

#### Compact detector description entry element

The `<detector>` tag defines a new instance of a detector and requires the 
attributes "id", "name", and "type". For example from part 1:

```xml
    <detector id="2" name="GEMTracker" vis="RedVis" type="my_GEMTracker" readout="GEMTrackerHits" > 
      <layer id="1" z="-100 *cm" inner_r="40*cm"  outer_r="120*cm"   phi0_offset="0.0*deg" />
      <layer id="2" z="-80 *cm"  inner_r="30*cm"  outer_r="90*cm"    phi0_offset="0.0*deg" />
      <layer id="3" z="-60 *cm"  inner_r="20*cm"  outer_r="70*cm"    phi0_offset="0.0*deg" />
      <layer id="4" z="-40 *cm"  inner_r="10*cm"  outer_r="20.0*cm"  phi0_offset="0.0*deg" />
      <layer id="5" z=" 40 *cm"  inner_r="10*cm"  outer_r="20.0*cm" phi0_offset="0.0*deg" />
      <layer id="6" z=" 60 *cm"  inner_r="25*cm"  outer_r="70.0*cm" phi0_offset="0.0*deg" />
      <layer id="7" z=" 80 *cm"  inner_r="30*cm"  outer_r="90.0*cm" phi0_offset="0.0*deg" />
      <layer id="8" z="100 *cm"  inner_r="40*cm"  outer_r="100.0*cm" phi0_offset="0.0*deg" />
    </detector>
````

This defines an instance of the detector named "GEMTracker" of type 
"my_GEMTracker" (i.e. the type-name given in the first argument of the DD4hep 
`DECLARE_DETELEMENT` macro) and with id=2. Each `<detector>` must have a unique id.
The additional attributes (vis, readout, zoffset) are optional.

The detector tag is provided as the second argument in the `build_detector` 
function. It can be parsed *how ever you want* in order to extract detector 
information. The allowed attributes are listed in 
[`UnicodeValues.h`](http://test-dd4hep.web.cern.ch/test-dd4hep/doxygen/html/_unicode_values_8h_source.html) 
where it is clear how to add new attributes.

#### Geometry Construction

If you are familiar with Geant4 or TGeo geometry construction then this will be 
easy. DD4hep has TGeo under hood but there are a few naming tweaks. The 
following table will help orient the user. 

| *DD4hep*       |   *Geant4*        | *TGeo |
| ------       | --------        | ---- |
| Solid        | G4VSolid        | TGeoShape |
| Volume       | G4LogicalVolume | TGeoVolume |
| PlacedVolume | G4PVPlacement   | TGeoNode |
| Element      | G4Element       | TGeoElement |
| Material     | G4Material      | TGeoMaterial/TGeoMedium|

##### XML Parsing Tip : Provide good default values

If you have a detector parameter which we later will tweak (while optimizing 
the design) try to get the value from the xml element but provide a good 
default value.  For example:

```cpp
double radius = ( x_det.hasAttr(_Unicode(radius)) ) ?  x_det.attr<double>(_Unicode(radius)) : 5.0*dd4hep::cm;
//or
double radius = dd4hep::getAttrOrDefault(x_det, _Unicode(radius), 5.0*dd4hep::cm)
```

This provides a default radius of 5 cm when `x_det` does not have a "radius" 
attribute defined. We will return to this later. 

####  Critical parts of build_detector


We will now look at the function in `src/TrapEndcapTracker_geo.cpp`.
Note that **this is not a good example to copy**, rather it is a simple example to walk through.

```cpp
static Ref_t create_detector(Detector& lcdd, xml_h e, SensitiveDetector sens)
{
  typedef vector<PlacedVolume> Placements;

  xml_det_t   x_det     = e;
  Material    air       = lcdd.air();
  Material    carbon    = lcdd.material("CarbonFiber");
  Material    silicon   = lcdd.material("SiliconOxide");
  int         det_id    = x_det.id();
  string      det_name  = x_det.nameStr();
  PlacedVolume             pv;

  DetElement  sdet(det_name, det_id);
  Assembly    assembly(det_name+"_assembly");

  sens.setType("tracker");
  string module_name = "GEM";

  double thickness = 0.01*dd4hep::cm;

  int N_layers      = 0;

  for(xml_coll_t lay( x_det, _U(layer) ); lay; ++lay, ++N_layers)  {
    xml_comp_t x_layer     = lay;
    double     inner_r     = x_layer.attr<double>(  _Unicode(inner_r) ) ;
    double     outer_r     = x_layer.attr<double>(  _Unicode(outer_r) ) ;
    double     phi0_offset = x_layer.attr<double>(  _Unicode(phi0_offset) ) ;
    double     z           = x_layer.attr<double>(  _Unicode(z) ) ;
    int        layer_id    = x_layer.id();//attr<double>(  _Unicode(z) ) ;
    string     layer_name  = std::string("gem_layer") + std::to_string(layer_id);
    
    Tube       gem_layer(inner_r, outer_r, thickness / 2.0);
    Volume     gem_layer_vol("gem_layer_vol", gem_layer, carbon);

    gem_layer_vol.setSensitiveDetector(sens);

    DetElement layer_DE( sdet, _toString(layer_id,"layer%d"), layer_id );

    pv = assembly.placeVolume( gem_layer_vol, Transform3D(RotationZ(phi0_offset),Position(0.0,0.0,z)) );
    pv.addPhysVolID( "layer", layer_id );
    layer_DE.setPlacement(pv);

  }

  sdet.setAttributes(lcdd, assembly,x_det.regionStr(),x_det.limitsStr(),x_det.visStr());

  pv = lcdd.pickMotherVolume(sdet).placeVolume(assembly);
  pv.addPhysVolID("system", det_id);      // Set the subdetector system ID.
  sdet.setPlacement(pv);

  assembly->GetShape()->ComputeBBox() ;
  return sdet;
}
```
Here is the XML again:
```xml
    <detector id="2" name="GEMTracker" vis="RedVis" type="my_GEMTracker" readout="GEMTrackerHits" > 
      <layer id="1" z="-100 *cm" inner_r="40*cm"  outer_r="120*cm"   phi0_offset="0.0*deg" />
      <layer id="2" z="-80 *cm"  inner_r="30*cm"  outer_r="90*cm"    phi0_offset="0.0*deg" />
      <layer id="3" z="-60 *cm"  inner_r="20*cm"  outer_r="70*cm"    phi0_offset="0.0*deg" />
      <layer id="4" z="-40 *cm"  inner_r="10*cm"  outer_r="20.0*cm"  phi0_offset="0.0*deg" />
      <layer id="5" z=" 40 *cm"  inner_r="10*cm"  outer_r="20.0*cm" phi0_offset="0.0*deg" />
      <layer id="6" z=" 60 *cm"  inner_r="25*cm"  outer_r="70.0*cm" phi0_offset="0.0*deg" />
      <layer id="7" z=" 80 *cm"  inner_r="30*cm"  outer_r="90.0*cm" phi0_offset="0.0*deg" />
      <layer id="8" z="100 *cm"  inner_r="40*cm"  outer_r="100.0*cm" phi0_offset="0.0*deg" />
    </detector>
````

What problems do you see with this construction?

- rigid
- hard coded
- no defaults
- single material
- lots more...

### What materials exist?

To browse the list of materials and elements (other than looking in the compact files), go to 
[geo viewer](https://eic.phy.anl.gov/geoviewer/index.htm?file=https://eicweb.phy.anl.gov/api/v4/projects/473/jobs/artifacts/master/raw/geo/detector_geo_full.root?job=report)
and scroll through the materials.
![browse_materials](../browse_materials.png)

Here we are grabbing the materials that are assumed to be already defined. Also 
we are getting the detector id and name defined in the `detector` tag.

Next we define an [Assembly 
volume](http://test-dd4hep.web.cern.ch/test-dd4hep/doxygen/html/classdd4hep_1_1_assembly.html).
Here we also stumble upon the important class 
[`dd4hep::DetElement`](http://test-dd4hep.web.cern.ch/test-dd4hep/doxygen/html/classdd4hep_1_1_det_element.html#details).
It is a means of providing the detector hierarchy/tree, but doesn't necessarily 
have to map exactly to detector geometry. However, it typically will typically 
parallel the geometry (and probably should).


## A more detailed GEM Tracker

Uncomment the include line in `gem_tracker.xml`


```xml
  <include ref="compact/gem_tracker_endcap.xml"/>
```
And have a look at the detector part of the compact file.

```xml
<detector
  id="GEMTrackerEndcap_ID"
  name="GEMTrackerEndcap"
  type="MyGEMTrackerEndcap"
  readout="GEMTrackerEndcapHits"
  vis="BlueVis"
  reflect="false">
  <module name="GEMModule1" vis="GreenVis">
    <trd x1="GEMTrackerEndcapFoilX1/2.0" x2="GEMTrackerEndcapFoilX2/2.0" z="GEMTrackerEndcapFoilY/2"/>
    <comment> Going from HV side to readout side</comment>
    <module_component thickness="0.127 * mm" material="Mylar"/>
    <module_component thickness="50.0*um" material="Kapton" name="entrance_window"/>
    <module_component thickness=" 3.0*mm" material="Ar10CO2" name="entrance region" />
    <module_component thickness="50.0*um" material="Kapton"/>
    <module_component thickness=" 3.0*um" material="Copper"/>
    <module_component thickness=" 3.0*mm" material="Ar10CO2" name="drift region"/>
    <module_component thickness="30.0*um" material="Kapton"  name="gem_foil"/>
    <module_component thickness=" 3.0*um" material="Copper"  name="gem_foil_Cu"/>
    <module_component thickness=" 2.0*mm" material="Ar10CO2" name="transfer region I"/>
    <module_component thickness="30.0*um" material="Kapton"  name="gem_foil"/>
    <module_component thickness=" 3.0*um" material="Copper"  name="gem_foil_Cu"/>
    <module_component thickness=" 2.0*mm" material="Ar10CO2" name="transfer region II"/>
    <module_component thickness="30.0*um" material="Kapton"  name="gem_foil"/>
    <module_component thickness=" 3.0*um" material="Copper"  name="gem_foil_Cu"/>
    <module_component thickness=" 2.0*mm" material="Ar10CO2" name="induction region"/>
    <module_component thickness="30.0*um" material="Kapton"  name="readout"     sensitive="true"/>
    <module_component thickness=" 3.0*um" material="Copper"  name="readout_Cu"/>
    <module_component thickness="127.0*um" material="Mylar"/>
    <module_component thickness="200.0*um" material="Epoxy" sensitive="true" vis="GreenVis"/>
  </module>
  <module name="GEMSupportModule1" vis="OrangeVis">
    <trd x1="GEMTrackerEndcapFoilX2/2.0" x2="GEMTrackerEndcapFoilX1/2.0" z="GEMTrackerEndcapFrameBotEdge_width"/>
    <module_component thickness="GEMTrackerEndcapFrame_thickness" material="Mylar"/>
  </module>
  <comment>
    GEMSupportModule2 is the support in between gem foils.
  </comment>
  <module name="GEMSupportModule2" vis="OrangeVis">
    <trd x1="GEMTrackerEndcapFrameSideEdge_width" x2="GEMTrackerEndcapFrameSideEdge_width" z="GEMTrackerEndcapFoilY/2"/>
    <module_component thickness="4.0*mm" material="Mylar"/>
  </module>
  <layer id="1" >
    <ring vis="PurpleVis"
      r="GEMTrackerEndcapFoil_rmin+GEMTrackerEndcapFoilY/2.0"
      zstart="GEMTrackerEndcap_zmin + 0.5*GEMTrackerEndcapLayer_thickness"
      nmodules="12" dz="10 * mm" module="GEMModule1" />
    <ring vis="PurpleVis" phi0="15.0*degree"
      r="GEMTrackerEndcapFoil_rmin+GEMTrackerEndcapFoilY/2.0"
      zstart="GEMTrackerEndcap_zmin + 0.5*GEMTrackerEndcapLayer_thickness"
      nmodules="12" dz="0 * mm" module="GEMSupportModule2" />
  </layer>
</detector>
```

Here you see a different strategy: build a module, then build layers constructed from the module. 


